<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Util\Environment;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Configurationmanager;
use Zeedhi\DevOps\Model\Devops\Entities;
use \Zeedhi\DevOps\Model\Servermanager;
use Util\Messages;
use Zeedhi\Framework\Log\Exception;

class Mnemonic {

    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var  \Zeedhi\DevOps\Model\Devops\Repositories\ServerMnemonic   */
    protected $serverMnemonicRepository;
    /** @var \Zeedhi\DevOps\Model\Devops\Repositories\ProductFilesMnemonic */
    protected $productFilesMnemonicRepository;
    /** @var \Zeedhi\DevOps\Model\Devops\Repositories\Mnemonic  */
    protected $mnemonicRepository;
    /** @var Environment */
    private $environment;
    /**
     * @var \Factory\Mnemonic
     */
    private $mnemonicFactory;
    /**
     * @var \Factory\ServerMnemonic
     */
    private $serverMnemonicFactory;


    const MNEMONIC_SEPARATOR = '@@';

    public function __construct(EntityManager $entityManager, Environment $environment, \Factory\Mnemonic $mnemonicFactory, \Factory\MnemonicGrouping $mnemonicGroupingFactory,
                                \Factory\ProductFilesMnemonic $productFilesMnemonicFactory, \Factory\ServerMnemonic $serverMnemonicFactory)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->mnemonicFactory = $mnemonicFactory;
        $this->mnemonicGroupingFactory = $mnemonicGroupingFactory;
        $this->productFilesMnemonicFactory = $productFilesMnemonicFactory;
        $this->serverMnemonicFactory = $serverMnemonicFactory;
        $this->mnemonicRepository = $this->entityManager->getRepository(Utilities::DEVOPS_ENTITY_DIR . 'Mnemonic');
        $this->mnemonicGroupingRepository = $this->entityManager->getRepository(Utilities::DEVOPS_ENTITY_DIR . 'MnemonicGrouping');
        $this->serverMnemonicRepository = $this->entityManager->getRepository(Utilities::DEVOPS_ENTITY_DIR . 'ServerMnemonic');
        $this->productFilesMnemonicRepository = $this->entityManager->getRepository(Utilities::DEVOPS_ENTITY_DIR . 'ProductFilesMnemonic');
    }

    public function saveMnemonic($mnemonic = null, $mnemonicString, $label = null){
        return $this->mnemonicFactory->saveMnemonic($mnemonic,$mnemonicString,$label);
    }

    public function createProductFilesMnemonic(Entities\Mnemonic $mnemonic,
                                               Configurationmanager\Entities\ProductFiles $productFile){
        return $this->productFilesMnemonicFactory->createProductFilesMnemonic($mnemonic, $productFile);
    }
    public function createServerMnemonic(Entities\Mnemonic $mnemonic,
                                         Servermanager\Entities\Server $server, $value){
        return $this->serverMnemonicFactory->createServerMnemonic($mnemonic, $server, $value);
    }
    
    public function updateServerMnemonic(Entities\ServerMnemonic $mnemonic, $value){
        return $this->serverMnemonicFactory->updateServerMnemonic($mnemonic, $value);
    }

    public function findMnemonicByProductFiles($productFileId, $filter = " "){
        return $this->mnemonicRepository->findMnemonicByProductFiles($productFileId, $filter);
    }
    
    public function findMnemonicGrouping($conditions, $page, $pageSize){
        $filters = Utilities::createQueryFiltered($conditions);
        $limit   = $page * $pageSize;
        $offset   = ($page - 1) * $pageSize;
        $organizationId = $this->environment->getOrganizationId();
        return $this->mnemonicGroupingRepository->findMnemonicGrouping($organizationId, $filters, $limit, $offset);
    }
    
    public function saveMnemonicGruping($id, $name){
        $mnemonicGroup = null;
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
        if($id) {
            $mnemonicGroup = $this->mnemonicGroupingRepository->find($id);
        }
        
        $mnemonicEntity = $this->mnemonicGroupingFactory->saveMnemonic($mnemonicGroup, $name);
        $this->entityManager->persist($mnemonicEntity);
        $this->entityManager->flush();
        $connection->commit();
         $organizationId = $this->environment->getOrganizationId();
         $filters = "AND ID = " . $mnemonicEntity->getId();
        return $this->mnemonicGroupingRepository->findMnemonicGrouping($organizationId, $filters, null, null);
    }

    public function saveMnemonicForProduct($mnemonicId, $label, $defaultValue, $isEncrypted, $mnemonicGrouping = null){
        /** @var Entities\Mnemonic $mnemonic */
        $mnemonic = $this->mnemonicRepository->find($mnemonicId);
        $mnemonicsServer = $this->serverMnemonicRepository->findBy(array('mnemonic' => $mnemonicId));
        $mnemonic->setLabel($label);
        $mnemonic->setMnemonicGroupingId($mnemonicGrouping);
        
        if(($mnemonic->getIsEncrypted() != $isEncrypted) && $isEncrypted == 'Y'){
            foreach($mnemonicsServer as $mnemonicServer ){
                $mnemonicServer->setValue(Utilities::encrypt($mnemonicServer->getValue()));
                $this->entityManager->persist($mnemonicServer);
            }
            $this->entityManager->flush();
        }
        else if (($mnemonic->getIsEncrypted() != $isEncrypted) && $isEncrypted == 'N'){
            foreach($mnemonicsServer as $mnemonicServer ){
                $mnemonicServer->setValue(Utilities::decrypt($mnemonicServer->getValue()));
                $this->entityManager->persist($mnemonicServer);
            }
            $this->entityManager->flush();
        }
        if($mnemonicId && $defaultValue == '**********'){
            if($isEncrypted == 'Y'){
                $mnemonic->setDefaultValue($mnemonic->getDefaultValue());
            }else {
                $mnemonic->setDefaultValue(Utilities::decrypt($mnemonic->getDefaultValue()));
            }
        }else if($isEncrypted == 'Y' && strtoupper($defaultValue) != 'NULL'){
            $mnemonic->setDefaultValue(Utilities::encrypt($defaultValue));
        }else {
            $mnemonic->setDefaultValue($defaultValue);
        }
        $mnemonic->setIsEncrypted($isEncrypted);
        $this->entityManager->persist($mnemonic);
        $this->entityManager->flush();

        return $mnemonic->getDefaultValue();
    }

    public function findMnemonicByProductFilesAndServer($productFileId, $serverId){
        return $this->mnemonicRepository->findMnemonicByProductFilesAndServer($productFileId, $serverId);
    }

    public function findMnemonicByServer($serverId, $filters, $filterWhereInByProductsId = "", $limit = null, $offset = null){
        return $this->mnemonicRepository->findMnemonicByServer($serverId, $filters, $filterWhereInByProductsId, $limit, $offset);
    }

    public function validateMnemonicWithLabelByProductFileId($productFileId){
        return $this->mnemonicRepository->validateMnemonicWithLabelByProductFileId($productFileId);
    }

    /**
     * @param Servermanager\Entities\ProductServer[] $productServers
     * @param Configurationmanager\Entities\ProductFiles[] $productFiles
     * @return mixed
     */
    public function validateDeployServerMnemonic(array $productServers, array $productFiles)
    {
        $productFileIds = Utilities::getArrayByArrayObjectWithGet($productFiles);
        $mnemonicsPerServer = array();
        foreach ($productServers as $productServer) {
            $server = $productServer->getServer();
            $this->setMessageValidateMnemonic($productFileIds, $server, $mnemonicsPerServer);
        }

        return $mnemonicsPerServer;
    }

    /**
     * @param Servermanager\Entities\Server $server
     * @param Configurationmanager\Entities\ProductFiles[] $productFiles
     * @return mixed
     */

    public function validateServerMnemonic( $server, array $productFiles)
    {
        $productFileIds = Utilities::getArrayByArrayObjectWithGet($productFiles);
        return $this->setMessageValidateMnemonic($productFileIds, $server, $mnemonicsPerServer);
    }

    public function saveMnemonicByServer(Servermanager\Entities\Server $server, Entities\Mnemonic $mnemonic = null,
                                         $mnemonicString, $value = null , $label = null, $isEncrypted = 'N', $mnemonicGrouping = null){

        $mnemonic->setMnemonicGroupingId($mnemonicGrouping);
        if(empty($mnemonic)){
            $mnemonic = $this->saveMnemonic(null, $mnemonicString,$label);
        }else{
            $mnemonic->setLabel($label);
        }
        
        $this->entityManager->persist($mnemonic);
         $this->entityManager->flush();
        /** @var Entities\ServerMnemonic $serverMnemonic */
        $serverMnemonic = $this->serverMnemonicRepository
            ->findOneBy(array('mnemonic' => $mnemonic, 'server' => $server));
        if (empty($serverMnemonic)) {
            $serverMnemonic = $this->createServerMnemonic($mnemonic, $server, $value);
            $this->entityManager->persist($serverMnemonic);
            $this->entityManager->flush();
        }
        if($isEncrypted == 'Y' && $value == '**********'){
            $serverMnemonic->setValue($serverMnemonic->getValue());
        }else if($isEncrypted == 'Y' && strtoupper($value) != 'NULL'){
            $serverMnemonic->setValue(Utilities::encrypt($value));
        } else {
            $serverMnemonic->setValue($value);
        }
        $this->entityManager->persist($serverMnemonic);
        $this->entityManager->flush();
        
        return $mnemonic->getId();
    }

    public function setMnemonicsProductFile($productFile, $allMnemonicString)
    {
        foreach ($allMnemonicString as $mnemonicString) {
            /** @var  $mnemonic */
            $mnemonic = $this->mnemonicRepository->findOneBy(array('mnemonic' => $mnemonicString));
            if (empty($mnemonic)) {
                $mnemonic = $this->saveMnemonic(null, $mnemonicString);
                $this->entityManager->persist($mnemonic);
                $this->entityManager->flush();
            }
            $mnemonicProductFile = $this->productFilesMnemonicRepository
                ->findOneBy(array('mnemonic' => $mnemonic, 'productFiles' => $productFile));
            if (empty($mnemonicProductFile)) {
                $mnemonicProductFile = $this->createProductFilesMnemonic($mnemonic, $productFile);
                $this->entityManager->persist($mnemonicProductFile);
                $this->entityManager->flush();
            }
        }
        $mnemonicIdNotInScript = $this->productFilesMnemonicRepository->findMnemonicNotInScript($allMnemonicString,$productFile->getId());
        $mnemonicIdNotInScript = Utilities::arrayAssociativeToArray($mnemonicIdNotInScript);
        $mnemonicProductFilesNotInScript = $this->productFilesMnemonicRepository->findBy(array('mnemonic' => $mnemonicIdNotInScript,'productFiles' => $productFile));
        foreach ($mnemonicProductFilesNotInScript as $mnemonicProductFile) {
            $this->entityManager->remove($mnemonicProductFile);
        }
        if(!empty($mnemonicProductFilesNotInScript)){
            $this->entityManager->flush();
        }
    }

    /**
     * @param $productFileIds
     * @param $server
     * @param $mnemonicsPerServer
     * @return array
     */
    protected function setMessageValidateMnemonic($productFileIds, Servermanager\Entities\Server $server, &$mnemonicsPerServer = array())
    {
        $mnemonicsPerServer = array();
        $mnemonics = $this->mnemonicRepository->findMnemonicNotValueByProductFilesAndServer(
            $productFileIds, $server->getId());
        if (!empty($mnemonics)) {
            $mnemonicPerServer['SERVER_NAME'] = $server->getName();
            $mnemonicPerServer['MNEMONICS'] = $mnemonics;
            $mnemonicsPerServer[] = $mnemonicPerServer;
        }
        return $mnemonicsPerServer;
    }
    
    public function deleteMnemonicGroup($id)
    {
        $mnemonicsInUse = $this->mnemonicGroupingRepository->validateIfMnemonicGroupIsInUse($id);
        if($mnemonicsInUse) {
            $mnemonicList = "";
            foreach($mnemonicsInUse as $mnemonicInUse){
                $mnemonicName = $mnemonicInUse["MNEMONIC"];
                $mnemonicList .= "<br> $mnemonicName";
            }
            throw new Exception(Messages::ERR_MNEMONIC_GROUP_CANNOT_BE_DELETED . $mnemonicList);
        }
        $mnemonicGroup = $this->mnemonicGroupingRepository->find($id);
        $this->entityManager->remove($mnemonicGroup);
        $this->entityManager->flush();
    }
}
