module.exports = function(grunt) {
    var utilPath = 'assets/js/util/*.js';
    var repositoryPath = 'assets/js/model/**/*.js';
    var servicePath = 'assets/js/services/**/*.js';
    var controllerPath = 'assets/js/controllers/*.js';
    var libFiles = [
        'zh-grid-navigation/dist/GridNavigationLibrary.js',

        'zh-select-autocomplete/dist/zh-select-autocomplete.js',
        'zh-memory-storage/src/*.js',
        'zh-cache-bypass/src/CacheBypass.js',
        'gitQueue/dist/main.min.js',
        'zhuaclfrontend/assets/dist/main.js',
        'zh-interval-component/dist/zh-interval-component.js',

        'zh-preferences/dist/preferences.js',
        'zh-filter-preferences/dist/filter-preferences.js',
        'zh-layout-preferences/dist/layout-preferences.js',
        'zh-view-preferences/dist/view-preferences.js',
        'zh-grid-configuration/dist/zh-grid-configuration.js',

        'zhulog/dist/main.js',
        'tek-libraries/dist/main.js'
    ];
    
    function getLibPaths() {
        return libFiles.map(function(fileName) {
            return 'bower_components/' + fileName;
        });
    }

    function getCSSPaths() {
        var paths = [];
        paths.push('./assets/css/**/*.css');
        paths.push('!./assets/css/app.css');
        paths.push('bower_components/zh-grid-navigation/dist/style.css');
        paths.push('bower_components/zh-preferences/dist/style/colors/styles-inflightor.css');
        paths.push('bower_components/zh-grid-configuration/dist/style/colors/styles-inflightor.css');
        paths.push('bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.css');
        return paths;
    }
    
    grunt.initConfig({
        concat: {
            zh: {
                options: {
                    process: function(src, filepath) {
                        return '\n' + '// FILE: ' + filepath + '\n' + src;
                    }
                },
                src: ['libs/**/*.js', utilPath, repositoryPath, servicePath, controllerPath],
                dest: 'dist/main.js'
            },
            css: {
                options: {
                    process: function (src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: getCSSPaths(),
                dest: './assets/css/app.css'
            },
            buildLibs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getLibPaths(),
                dest: 'dist/libs.js'
            }
        },
        jshint: {
            options: {
                reporterOutput: ''  
            },
            all: ['Gruntfile.js', 'lib/**/*.js', utilPath, repositoryPath, servicePath, controllerPath]
        },
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'dist/main.min.js': ['libs/**/*.js', utilPath, repositoryPath, servicePath, controllerPath]
                }
            }
        },
        watch: {
            pivotal: {
                files: [utilPath, repositoryPath, servicePath, controllerPath],
                tasks: ['jshint', 'uglify', 'concat']
            }
        },
        zhIdGenerator: {
            options: {
                jsonPath: ["./assets/json"],
                menuPath: ["./assets/json/menu.json"],
                replaceIds: false
            }
        },
        buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        buildEnvironment: {
            development: {
                configFile: "./config/template/devEnvironment.json"
            },
            production: {
                configFile: "./config/template/prodEnvironment.json"
            },
            test: {
                configFile: "./config/template/testEnvironment.json"
            },
            srcFolders: ["../backend/config", "./config", "./", "./assets/js", "./assets/json"]
        }
    });

    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-environment-builder');

    // Default task.
    grunt.registerTask('default', ['jshint', 'uglify', 'concat', 'buildAclStruct', 'buildEnvironment']);
    grunt.registerTask('buildLibs', ['concat:buildLibs']);
    grunt.registerTask('commit', ['jshint', 'uglify', 'concat']);
};